@@ -44,13 +44,14 @@ class TeleBot:
         getUpdates
     """
 
-    def __init__(self, token, threaded=True):
+    def __init__(self, token, threaded=True, skip_pending=False):
         """
         :param token: bot API token
         :return: Telebot object.
         """
         self.token = token
         self.update_listener = []
+        self.skip_pending = skip_pending
 
         self.__stop_polling = threading.Event()
         self.last_update_id = 0
@@ -65,6 +66,8 @@ class TeleBot:
         self.pre_message_subscribers_next_step = {}
 
         self.message_handlers = []
+        self.inline_handlers = []
+        self.chosen_inline_handlers = []
 
         self.threaded = threaded
         if self.threaded:
@@ -90,29 +93,67 @@ class TeleBot:
             ret.append(types.Update.de_json(ju))
         return ret
 
+    def __skip_updates(self):
+        """
+        Get and discard all pending updates before first poll of the bot
+        :return: total updates skipped
+        """
+        total = 0
+        updates = self.get_updates(offset=self.last_update_id, timeout=1)
+        while updates:
+            total += len(updates)
+            for update in updates:
+                if update.update_id > self.last_update_id:
+                    self.last_update_id = update.update_id
+            updates = self.get_updates(offset=self.last_update_id + 1, timeout=1)
+        return total
+
     def __retrieve_updates(self, timeout=20):
         """
         Retrieves any updates from the Telegram API.
         Registered listeners and applicable message handlers will be notified when a new message arrives.
         :raises ApiException when a call has failed.
         """
+        if self.skip_pending:
+            logger.debug('Skipped {0} pending messages'.format(self.__skip_updates()))
+            self.skip_pending = False
         updates = self.get_updates(offset=(self.last_update_id + 1), timeout=timeout)
+        self.process_new_updates(updates)
+
+    def process_new_updates(self, updates):
         new_messages = []
+        new_inline_querys = []
+        new_chosen_inline_results = []
         for update in updates:
             if update.update_id > self.last_update_id:
                 self.last_update_id = update.update_id
-            new_messages.append(update.message)
-        logger.debug('Received {0} new messages'.format(len(new_messages)))
+            if update.message:
+                new_messages.append(update.message)
+            if update.inline_query:
+                new_inline_querys.append(update.inline_query)
+            if update.chosen_inline_result:
+                new_chosen_inline_results.append(update.chosen_inline_result)
+        logger.debug('Received {0} new updates'.format(len(updates)))
         if len(new_messages) > 0:
             self.process_new_messages(new_messages)
+        if len(new_inline_querys) > 0:
+            self.process_new_inline_query(new_inline_querys)
+        if len(new_chosen_inline_results) > 0:
+            self.process_new_chosen_inline_query(new_chosen_inline_results)
 
     def process_new_messages(self, new_messages):
         self._append_pre_next_step_handler()
         self.__notify_update(new_messages)
-        self._notify_command_handlers(new_messages)
+        self._notify_command_handlers(self.message_handlers, new_messages)
         self._notify_message_subscribers(new_messages)
         self._notify_message_next_handler(new_messages)
 
+    def process_new_inline_query(self, new_inline_querys):
+        self._notify_command_handlers(self.inline_handlers, new_inline_querys)
+
+    def process_new_chosen_inline_query(self, new_chosen_inline_querys):
+        self._notify_command_handlers(self.chosen_inline_handlers, new_chosen_inline_querys)
+
     def __notify_update(self, new_messages):
         for listener in self.update_listener:
             self.__exec_task(listener, new_messages)
@@ -236,7 +277,7 @@ class TeleBot:
         return types.UserProfilePhotos.de_json(result)
 
     def send_message(self, chat_id, text, disable_web_page_preview=None, reply_to_message_id=None, reply_markup=None,
-                     parse_mode=None):
+                     parse_mode=None, disable_notification=None):
         """
         Use this method to send text messages.
 
@@ -249,23 +290,27 @@ class TeleBot:
         :param reply_to_message_id:
         :param reply_markup:
         :param parse_mode:
+        :param disable_notification: Boolean, Optional. Sends the message silently.
         :return: API reply.
         """
         return types.Message.de_json(
             apihelper.send_message(self.token, chat_id, text, disable_web_page_preview, reply_to_message_id,
-                                   reply_markup, parse_mode))
+                                   reply_markup, parse_mode, disable_notification))
 
-    def forward_message(self, chat_id, from_chat_id, message_id):
+    def forward_message(self, chat_id, from_chat_id, message_id, disable_notification=None):
         """
         Use this method to forward messages of any kind.
+        :param disable_notification:
         :param chat_id: which chat to forward
         :param from_chat_id: which chat message from
         :param message_id: message id
         :return: API reply.
         """
-        return types.Message.de_json(apihelper.forward_message(self.token, chat_id, from_chat_id, message_id))
+        return types.Message.de_json(
+            apihelper.forward_message(self.token, chat_id, from_chat_id, message_id, disable_notification))
 
-    def send_photo(self, chat_id, photo, caption=None, reply_to_message_id=None, reply_markup=None):
+    def send_photo(self, chat_id, photo, caption=None, reply_to_message_id=None, reply_markup=None,
+                   disable_notification=None):
         """
         Use this method to send photos.
         :param chat_id:
@@ -276,10 +321,11 @@ class TeleBot:
         :return: API reply.
         """
         return types.Message.de_json(
-            apihelper.send_photo(self.token, chat_id, photo, caption, reply_to_message_id, reply_markup))
+            apihelper.send_photo(self.token, chat_id, photo, caption, reply_to_message_id, reply_markup,
+                                 disable_notification))
 
     def send_audio(self, chat_id, audio, duration=None, performer=None, title=None, reply_to_message_id=None,
-                   reply_markup=None):
+                   reply_markup=None, disable_notification=None):
         """
         Use this method to send audio files, if you want Telegram clients to display them in the music player. Your audio must be in the .mp3 format.
         :param chat_id:Unique identifier for the message recipient
@@ -293,9 +339,10 @@ class TeleBot:
         """
         return types.Message.de_json(
             apihelper.send_audio(self.token, chat_id, audio, duration, performer, title, reply_to_message_id,
-                                 reply_markup))
+                                 reply_markup, disable_notification))
 
-    def send_voice(self, chat_id, voice, duration=None, reply_to_message_id=None, reply_markup=None):
+    def send_voice(self, chat_id, voice, duration=None, reply_to_message_id=None, reply_markup=None,
+                   disable_notification=None):
         """
         Use this method to send audio files, if you want Telegram clients to display the file as a playable voice message.
         :param chat_id:Unique identifier for the message recipient.
@@ -306,9 +353,10 @@ class TeleBot:
         :return: Message
         """
         return types.Message.de_json(
-            apihelper.send_voice(self.token, chat_id, voice, duration, reply_to_message_id, reply_markup))
+            apihelper.send_voice(self.token, chat_id, voice, duration, reply_to_message_id, reply_markup,
+                                 disable_notification))
 
-    def send_document(self, chat_id, data, reply_to_message_id=None, reply_markup=None):
+    def send_document(self, chat_id, data, reply_to_message_id=None, reply_markup=None, disable_notification=None):
         """
         Use this method to send general files.
         :param chat_id:
@@ -318,9 +366,10 @@ class TeleBot:
         :return: API reply.
         """
         return types.Message.de_json(
-            apihelper.send_data(self.token, chat_id, data, 'document', reply_to_message_id, reply_markup))
+            apihelper.send_data(self.token, chat_id, data, 'document', reply_to_message_id, reply_markup,
+                                disable_notification))
 
-    def send_sticker(self, chat_id, data, reply_to_message_id=None, reply_markup=None):
+    def send_sticker(self, chat_id, data, reply_to_message_id=None, reply_markup=None, disable_notification=None):
         """
         Use this method to send .webp stickers.
         :param chat_id:
@@ -330,9 +379,11 @@ class TeleBot:
         :return: API reply.
         """
         return types.Message.de_json(
-            apihelper.send_data(self.token, chat_id, data, 'sticker', reply_to_message_id, reply_markup))
+            apihelper.send_data(self.token, chat_id, data, 'sticker', reply_to_message_id, reply_markup,
+                                disable_notification))
 
-    def send_video(self, chat_id, data, duration=None, caption=None, reply_to_message_id=None, reply_markup=None):
+    def send_video(self, chat_id, data, duration=None, caption=None, reply_to_message_id=None, reply_markup=None,
+                   disable_notification=None):
         """
         Use this method to send video files, Telegram clients support mp4 videos.
         :param chat_id: Integer : Unique identifier for the message recipient — User or GroupChat id
@@ -344,9 +395,11 @@ class TeleBot:
         :return:
         """
         return types.Message.de_json(
-            apihelper.send_video(self.token, chat_id, data, duration, caption, reply_to_message_id, reply_markup))
+            apihelper.send_video(self.token, chat_id, data, duration, caption, reply_to_message_id, reply_markup,
+                                 disable_notification))
 
-    def send_location(self, chat_id, latitude, longitude, reply_to_message_id=None, reply_markup=None):
+    def send_location(self, chat_id, latitude, longitude, reply_to_message_id=None, reply_markup=None,
+                      disable_notification=None):
         """
         Use this method to send point on the map.
         :param chat_id:
@@ -357,7 +410,8 @@ class TeleBot:
         :return: API reply.
         """
         return types.Message.de_json(
-            apihelper.send_location(self.token, chat_id, latitude, longitude, reply_to_message_id, reply_markup))
+            apihelper.send_location(self.token, chat_id, latitude, longitude, reply_to_message_id, reply_markup,
+                                    disable_notification))
 
     def send_chat_action(self, chat_id, action):
         """
@@ -377,6 +431,19 @@ class TeleBot:
         """
         return self.send_message(message.chat.id, text, reply_to_message_id=message.message_id, **kwargs)
 
+    def answer_inline_query(self, inline_query_id, results, cache_time=None, is_personal=None, next_offset=None):
+        """
+        Use this method to send answers to an inline query. On success, True is returned.
+        No more than 50 results per query are allowed.
+        :param inline_query_id: Unique identifier for the answered query
+        :param results: Array of results for the inline query
+        :param cache_time: The maximum amount of time in seconds that the result of the inline query may be cached on the server.
+        :param is_personal: Pass True, if results may be cached on the server side only for the user that sent the query.
+        :param next_offset: Pass the offset that a client should send in the next query with the same text to receive more results.
+        :return: True means success.
+        """
+        return apihelper.answer_inline_query(self.token, inline_query_id, results, cache_time, is_personal, next_offset)
+
     def register_for_reply(self, message, callback):
         """
         Registers a callback function to be notified when a reply to `message` arrives.
@@ -397,7 +464,7 @@ class TeleBot:
 
     def _notify_message_subscribers(self, new_messages):
         for message in new_messages:
-            if not hasattr(message, 'reply_to_message'):
+            if not message.reply_to_message:
                 continue
 
             reply_msg_id = message.reply_to_message.message_id
@@ -470,21 +537,65 @@ class TeleBot:
         :param content_types: This commands' supported content types. Must be a list. Defaults to ['text'].
         """
 
-        def decorator(fn):
-            handler_dict = {'function': fn}
-            filters = {'content_types': content_types}
-            if regexp:
-                filters['regexp'] = regexp
-            if func:
-                filters['lambda'] = func
-            if commands:
-                filters['commands'] = commands
-            handler_dict['filters'] = filters
-            self.message_handlers.append(handler_dict)
-            return fn
+        def decorator(handler):
+            self.add_message_handler(handler, commands, regexp, func, content_types)
+            return handler
 
         return decorator
 
+    def add_message_handler(self, handler, commands=None, regexp=None, func=None, content_types=None):
+        if content_types is None:
+            content_types = ['text']
+
+        filters = {'content_types': content_types}
+        if regexp:
+            filters['regexp'] = regexp
+        if func:
+            filters['lambda'] = func
+        if commands:
+            filters['commands'] = commands
+
+        handler_dict = {
+            'function': handler,
+            'filters': filters
+        }
+
+        self.message_handlers.append(handler_dict)
+
+    def inline_handler(self, func):
+        def decorator(handler):
+            self.add_inline_handler(handler, func)
+            return handler
+
+        return decorator
+
+    def add_inline_handler(self, handler, func):
+        filters = {'lambda': func}
+
+        handler_dict = {
+            'function': handler,
+            'filters': filters
+        }
+
+        self.inline_handlers.append(handler_dict)
+
+    def chosen_inline_handler(self, func):
+        def decorator(handler):
+            self.add_chosen_inline_handler(handler, func)
+            return handler
+
+        return decorator
+
+    def add_chosen_inline_handler(self, handler, func):
+        filters = {'lambda': func}
+
+        handler_dict = {
+            'function': handler,
+            'filters': filters
+        }
+
+        self.chosen_inline_handlers.append(handler_dict)
+
     @staticmethod
     def _test_message_handler(message_handler, message):
         for filter, filter_value in six.iteritems(message_handler['filters']):
@@ -504,9 +615,9 @@ class TeleBot:
             return filter_value(message)
         return False
 
-    def _notify_command_handlers(self, new_messages):
+    def _notify_command_handlers(self, handlers, new_messages):
         for message in new_messages:
-            for message_handler in self.message_handlers:
+            for message_handler in handlers:
                 if self._test_message_handler(message_handler, message):
                     self.__exec_task(message_handler['function'], message)
                     break
